@@ -351,7 +351,9 @@ nlm_swap_auxiliary_headers_in (bfd *abfd)
 	      bfd_byte *contents;
 	      bfd_byte *p, *pend;
 
-	      BFD_ASSERT (hdrLength == 0 && hdr == NULL);
+	      /* See PR 21840 for a reproducer.  */
+	      if (hdrLength != 0 || hdr != NULL)
+		return FALSE;
 
 	      pos = bfd_tell (abfd);
 	      if (bfd_seek (abfd, dataOffset, SEEK_SET) != 0)
@@ -1573,7 +1575,7 @@ nlm_external_reloc_compare (const void *p1, const void *p2)
      code sections
      data sections
      other sections (custom data, messages, help, shared NLM, RPC,
-     		     module dependencies)
+		     module dependencies)
      relocation fixups
      external references (imports)
      public symbols (exports)
