@@ -1,5 +1,5 @@
 /* Generic target-file-type support for the BFD library.
-   Copyright (C) 1990-2015 Free Software Foundation, Inc.
+   Copyright (C) 1990-2017 Free Software Foundation, Inc.
    Written by Cygnus Support.
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -144,6 +144,7 @@ DESCRIPTION
 
 .enum bfd_flavour
 .{
+.  {* N.B. Update bfd_flavour_name if you change this.  *}
 .  bfd_target_unknown_flavour,
 .  bfd_target_aout_flavour,
 .  bfd_target_coff_flavour,
@@ -218,32 +219,33 @@ DESCRIPTION
 .  {* Entries for byte swapping for data. These are different from the
 .     other entry points, since they don't take a BFD as the first argument.
 .     Certain other handlers could do the same.  *}
-.  bfd_uint64_t   (*bfd_getx64) (const void *);
-.  bfd_int64_t    (*bfd_getx_signed_64) (const void *);
-.  void           (*bfd_putx64) (bfd_uint64_t, void *);
-.  bfd_vma        (*bfd_getx32) (const void *);
+.  bfd_uint64_t	  (*bfd_getx64) (const void *);
+.  bfd_int64_t	  (*bfd_getx_signed_64) (const void *);
+.  void		  (*bfd_putx64) (bfd_uint64_t, void *);
+.  bfd_vma	  (*bfd_getx32) (const void *);
 .  bfd_signed_vma (*bfd_getx_signed_32) (const void *);
-.  void           (*bfd_putx32) (bfd_vma, void *);
-.  bfd_vma        (*bfd_getx16) (const void *);
+.  void		  (*bfd_putx32) (bfd_vma, void *);
+.  bfd_vma	  (*bfd_getx16) (const void *);
 .  bfd_signed_vma (*bfd_getx_signed_16) (const void *);
-.  void           (*bfd_putx16) (bfd_vma, void *);
+.  void		  (*bfd_putx16) (bfd_vma, void *);
 .
 .  {* Byte swapping for the headers.  *}
-.  bfd_uint64_t   (*bfd_h_getx64) (const void *);
-.  bfd_int64_t    (*bfd_h_getx_signed_64) (const void *);
-.  void           (*bfd_h_putx64) (bfd_uint64_t, void *);
-.  bfd_vma        (*bfd_h_getx32) (const void *);
+.  bfd_uint64_t	  (*bfd_h_getx64) (const void *);
+.  bfd_int64_t	  (*bfd_h_getx_signed_64) (const void *);
+.  void		  (*bfd_h_putx64) (bfd_uint64_t, void *);
+.  bfd_vma	  (*bfd_h_getx32) (const void *);
 .  bfd_signed_vma (*bfd_h_getx_signed_32) (const void *);
-.  void           (*bfd_h_putx32) (bfd_vma, void *);
-.  bfd_vma        (*bfd_h_getx16) (const void *);
+.  void		  (*bfd_h_putx32) (bfd_vma, void *);
+.  bfd_vma	  (*bfd_h_getx16) (const void *);
 .  bfd_signed_vma (*bfd_h_getx_signed_16) (const void *);
-.  void           (*bfd_h_putx16) (bfd_vma, void *);
+.  void		  (*bfd_h_putx16) (bfd_vma, void *);
 .
 .  {* Format dependent routines: these are vectors of entry points
 .     within the target vector structure, one for each format to check.  *}
 .
 .  {* Check the format of a file being read.  Return a <<bfd_target *>> or zero.  *}
-.  const struct bfd_target *(*_bfd_check_format[bfd_type_end]) (bfd *);
+.  const struct bfd_target *
+.	       (*_bfd_check_format[bfd_type_end]) (bfd *);
 .
 .  {* Set the format of a file being written.  *}
 .  bfd_boolean (*_bfd_set_format[bfd_type_end]) (bfd *);
@@ -269,10 +271,11 @@ BFD_JUMP_TABLE macros.
 .  {* Called when a new section is created.  *}
 .  bfd_boolean (*_new_section_hook) (bfd *, sec_ptr);
 .  {* Read the contents of a section.  *}
-.  bfd_boolean (*_bfd_get_section_contents)
-.    (bfd *, sec_ptr, void *, file_ptr, bfd_size_type);
-.  bfd_boolean (*_bfd_get_section_contents_in_window)
-.    (bfd *, sec_ptr, bfd_window *, file_ptr, bfd_size_type);
+.  bfd_boolean (*_bfd_get_section_contents) (bfd *, sec_ptr, void *, file_ptr,
+.					     bfd_size_type);
+.  bfd_boolean (*_bfd_get_section_contents_in_window) (bfd *, sec_ptr,
+.						       bfd_window *, file_ptr,
+.						       bfd_size_type);
 .
 .  {* Entry points to copy private data.  *}
 .#define BFD_JUMP_TABLE_COPY(NAME) \
@@ -290,25 +293,26 @@ BFD_JUMP_TABLE macros.
 .  bfd_boolean (*_bfd_copy_private_bfd_data) (bfd *, bfd *);
 .  {* Called to merge BFD general private data from one object file
 .     to a common output file when linking.  *}
-.  bfd_boolean (*_bfd_merge_private_bfd_data) (bfd *, bfd *);
+.  bfd_boolean (*_bfd_merge_private_bfd_data) (bfd *, struct bfd_link_info *);
 .  {* Called to initialize BFD private section data from one object file
 .     to another.  *}
 .#define bfd_init_private_section_data(ibfd, isec, obfd, osec, link_info) \
-.  BFD_SEND (obfd, _bfd_init_private_section_data, (ibfd, isec, obfd, osec, link_info))
-.  bfd_boolean (*_bfd_init_private_section_data)
-.    (bfd *, sec_ptr, bfd *, sec_ptr, struct bfd_link_info *);
+.	BFD_SEND (obfd, _bfd_init_private_section_data, \
+.		  (ibfd, isec, obfd, osec, link_info))
+.  bfd_boolean (*_bfd_init_private_section_data) (bfd *, sec_ptr, bfd *,
+.						  sec_ptr,
+.						  struct bfd_link_info *);
 .  {* Called to copy BFD private section data from one object file
 .     to another.  *}
-.  bfd_boolean (*_bfd_copy_private_section_data)
-.    (bfd *, sec_ptr, bfd *, sec_ptr);
+.  bfd_boolean (*_bfd_copy_private_section_data) (bfd *, sec_ptr, bfd *,
+.						  sec_ptr);
 .  {* Called to copy BFD private symbol data from one symbol
 .     to another.  *}
-.  bfd_boolean (*_bfd_copy_private_symbol_data)
-.    (bfd *, asymbol *, bfd *, asymbol *);
+.  bfd_boolean (*_bfd_copy_private_symbol_data) (bfd *, asymbol *, bfd *,
+.						 asymbol *);
 .  {* Called to copy BFD private header data from one object file
 .     to another.  *}
-.  bfd_boolean (*_bfd_copy_private_header_data)
-.    (bfd *, bfd *);
+.  bfd_boolean (*_bfd_copy_private_header_data) (bfd *, bfd *);
 .  {* Called to set private backend flags.  *}
 .  bfd_boolean (*_bfd_set_private_flags) (bfd *, flagword);
 .
@@ -323,9 +327,9 @@ BFD_JUMP_TABLE macros.
 .  NAME##_core_file_pid
 .
 .  char *      (*_core_file_failing_command) (bfd *);
-.  int         (*_core_file_failing_signal) (bfd *);
+.  int	       (*_core_file_failing_signal) (bfd *);
 .  bfd_boolean (*_core_file_matches_executable_p) (bfd *, bfd *);
-.  int         (*_core_file_pid) (bfd *);
+.  int	       (*_core_file_pid) (bfd *);
 .
 .  {* Archive entry points.  *}
 .#define BFD_JUMP_TABLE_ARCHIVE(NAME) \
@@ -343,17 +347,19 @@ BFD_JUMP_TABLE macros.
 .
 .  bfd_boolean (*_bfd_slurp_armap) (bfd *);
 .  bfd_boolean (*_bfd_slurp_extended_name_table) (bfd *);
-.  bfd_boolean (*_bfd_construct_extended_name_table)
-.    (bfd *, char **, bfd_size_type *, const char **);
-.  void        (*_bfd_truncate_arname) (bfd *, const char *, char *);
-.  bfd_boolean (*write_armap)
-.    (bfd *, unsigned int, struct orl *, unsigned int, int);
+.  bfd_boolean (*_bfd_construct_extended_name_table) (bfd *, char **,
+.						      bfd_size_type *,
+.						      const char **);
+.  void	       (*_bfd_truncate_arname) (bfd *, const char *, char *);
+.  bfd_boolean (*write_armap) (bfd *, unsigned int, struct orl *,
+.			       unsigned int, int);
 .  void *      (*_bfd_read_ar_hdr_fn) (bfd *);
 .  bfd_boolean (*_bfd_write_ar_hdr_fn) (bfd *, bfd *);
 .  bfd *       (*openr_next_archived_file) (bfd *, bfd *);
-.#define bfd_get_elt_at_index(b,i) BFD_SEND (b, _bfd_get_elt_at_index, (b,i))
+.#define bfd_get_elt_at_index(b,i) \
+.	BFD_SEND (b, _bfd_get_elt_at_index, (b,i))
 .  bfd *       (*_bfd_get_elt_at_index) (bfd *, symindex);
-.  int         (*_bfd_stat_arch_elt) (bfd *, struct stat *);
+.  int	       (*_bfd_stat_arch_elt) (bfd *, struct stat *);
 .  bfd_boolean (*_bfd_update_armap_timestamp) (bfd *);
 .
 .  {* Entry points used for symbols.  *}
@@ -374,71 +380,74 @@ BFD_JUMP_TABLE macros.
 .  NAME##_read_minisymbols, \
 .  NAME##_minisymbol_to_symbol
 .
-.  long        (*_bfd_get_symtab_upper_bound) (bfd *);
-.  long        (*_bfd_canonicalize_symtab)
-.    (bfd *, struct bfd_symbol **);
+.  long	       (*_bfd_get_symtab_upper_bound) (bfd *);
+.  long	       (*_bfd_canonicalize_symtab) (bfd *, struct bfd_symbol **);
 .  struct bfd_symbol *
-.              (*_bfd_make_empty_symbol) (bfd *);
-.  void        (*_bfd_print_symbol)
-.    (bfd *, void *, struct bfd_symbol *, bfd_print_symbol_type);
-.#define bfd_print_symbol(b,p,s,e) BFD_SEND (b, _bfd_print_symbol, (b,p,s,e))
-.  void        (*_bfd_get_symbol_info)
-.    (bfd *, struct bfd_symbol *, symbol_info *);
-.#define bfd_get_symbol_info(b,p,e) BFD_SEND (b, _bfd_get_symbol_info, (b,p,e))
-.  const char *(*_bfd_get_symbol_version_string)
-.    (bfd *, struct bfd_symbol *, bfd_boolean *);
-.#define bfd_get_symbol_version_string(b,s,h) BFD_SEND (b, _bfd_get_symbol_version_string, (b,s,h))
+.	       (*_bfd_make_empty_symbol) (bfd *);
+.  void	       (*_bfd_print_symbol) (bfd *, void *, struct bfd_symbol *,
+.				     bfd_print_symbol_type);
+.#define bfd_print_symbol(b,p,s,e) \
+.	BFD_SEND (b, _bfd_print_symbol, (b,p,s,e))
+.  void	       (*_bfd_get_symbol_info) (bfd *, struct bfd_symbol *,
+.					symbol_info *);
+.#define bfd_get_symbol_info(b,p,e) \
+.	BFD_SEND (b, _bfd_get_symbol_info, (b,p,e))
+.  const char *(*_bfd_get_symbol_version_string) (bfd *, struct bfd_symbol *,
+.						  bfd_boolean *);
+.#define bfd_get_symbol_version_string(b,s,h) \
+.	BFD_SEND (b, _bfd_get_symbol_version_string, (b,s,h))
 .  bfd_boolean (*_bfd_is_local_label_name) (bfd *, const char *);
 .  bfd_boolean (*_bfd_is_target_special_symbol) (bfd *, asymbol *);
 .  alent *     (*_get_lineno) (bfd *, struct bfd_symbol *);
-.  bfd_boolean (*_bfd_find_nearest_line)
-.    (bfd *, struct bfd_symbol **, struct bfd_section *, bfd_vma,
-.     const char **, const char **, unsigned int *, unsigned int *);
-.  bfd_boolean (*_bfd_find_line)
-.    (bfd *, struct bfd_symbol **, struct bfd_symbol *,
-.     const char **, unsigned int *);
+.  bfd_boolean (*_bfd_find_nearest_line) (bfd *, struct bfd_symbol **,
+.					  struct bfd_section *, bfd_vma,
+.					  const char **, const char **,
+.					  unsigned int *, unsigned int *);
+.  bfd_boolean (*_bfd_find_line) (bfd *, struct bfd_symbol **,
+.				  struct bfd_symbol *, const char **,
+.				  unsigned int *);
 .  bfd_boolean (*_bfd_find_inliner_info)
 .    (bfd *, const char **, const char **, unsigned int *);
 . {* Back-door to allow format-aware applications to create debug symbols
 .    while using BFD for everything else.  Currently used by the assembler
 .    when creating COFF files.  *}
-.  asymbol *   (*_bfd_make_debug_symbol)
-.    (bfd *, void *, unsigned long size);
+.  asymbol *   (*_bfd_make_debug_symbol) (bfd *, void *, unsigned long size);
 .#define bfd_read_minisymbols(b, d, m, s) \
-.  BFD_SEND (b, _read_minisymbols, (b, d, m, s))
-.  long        (*_read_minisymbols)
-.    (bfd *, bfd_boolean, void **, unsigned int *);
+.	BFD_SEND (b, _read_minisymbols, (b, d, m, s))
+.  long	       (*_read_minisymbols) (bfd *, bfd_boolean, void **,
+.				     unsigned int *);
 .#define bfd_minisymbol_to_symbol(b, d, m, f) \
-.  BFD_SEND (b, _minisymbol_to_symbol, (b, d, m, f))
-.  asymbol *   (*_minisymbol_to_symbol)
-.    (bfd *, bfd_boolean, const void *, asymbol *);
+.	BFD_SEND (b, _minisymbol_to_symbol, (b, d, m, f))
+.  asymbol *   (*_minisymbol_to_symbol) (bfd *, bfd_boolean, const void *,
+.					 asymbol *);
 .
 .  {* Routines for relocs.  *}
 .#define BFD_JUMP_TABLE_RELOCS(NAME) \
 .  NAME##_get_reloc_upper_bound, \
 .  NAME##_canonicalize_reloc, \
+.  NAME##_set_reloc, \
 .  NAME##_bfd_reloc_type_lookup, \
 .  NAME##_bfd_reloc_name_lookup
 .
-.  long        (*_get_reloc_upper_bound) (bfd *, sec_ptr);
-.  long        (*_bfd_canonicalize_reloc)
-.    (bfd *, sec_ptr, arelent **, struct bfd_symbol **);
+.  long	       (*_get_reloc_upper_bound) (bfd *, sec_ptr);
+.  long	       (*_bfd_canonicalize_reloc) (bfd *, sec_ptr, arelent **,
+.					   struct bfd_symbol **);
+.  void	       (*_bfd_set_reloc) (bfd *, sec_ptr, arelent **, unsigned int);
 .  {* See documentation on reloc types.  *}
 .  reloc_howto_type *
-.              (*reloc_type_lookup) (bfd *, bfd_reloc_code_real_type);
+.	       (*reloc_type_lookup) (bfd *, bfd_reloc_code_real_type);
 .  reloc_howto_type *
-.              (*reloc_name_lookup) (bfd *, const char *);
-.
+.	       (*reloc_name_lookup) (bfd *, const char *);
 .
 .  {* Routines used when writing an object file.  *}
 .#define BFD_JUMP_TABLE_WRITE(NAME) \
 .  NAME##_set_arch_mach, \
 .  NAME##_set_section_contents
 .
-.  bfd_boolean (*_bfd_set_arch_mach)
-.    (bfd *, enum bfd_architecture, unsigned long);
-.  bfd_boolean (*_bfd_set_section_contents)
-.    (bfd *, sec_ptr, const void *, file_ptr, bfd_size_type);
+.  bfd_boolean (*_bfd_set_arch_mach) (bfd *, enum bfd_architecture,
+.				      unsigned long);
+.  bfd_boolean (*_bfd_set_section_contents) (bfd *, sec_ptr, const void *,
+.					     file_ptr, bfd_size_type);
 .
 .  {* Routines used by the linker.  *}
 .#define BFD_JUMP_TABLE_LINK(NAME) \
@@ -451,39 +460,44 @@ BFD_JUMP_TABLE macros.
 .  NAME##_bfd_copy_link_hash_symbol_type, \
 .  NAME##_bfd_final_link, \
 .  NAME##_bfd_link_split_section, \
+.  NAME##_bfd_link_check_relocs, \
 .  NAME##_bfd_gc_sections, \
 .  NAME##_bfd_lookup_section_flags, \
 .  NAME##_bfd_merge_sections, \
 .  NAME##_bfd_is_group_section, \
 .  NAME##_bfd_discard_group, \
 .  NAME##_section_already_linked, \
-.  NAME##_bfd_define_common_symbol
+.  NAME##_bfd_define_common_symbol, \
+.  NAME##_bfd_define_start_stop
 .
-.  int         (*_bfd_sizeof_headers) (bfd *, struct bfd_link_info *);
-.  bfd_byte *  (*_bfd_get_relocated_section_contents)
-.    (bfd *, struct bfd_link_info *, struct bfd_link_order *,
-.     bfd_byte *, bfd_boolean, struct bfd_symbol **);
+.  int	       (*_bfd_sizeof_headers) (bfd *, struct bfd_link_info *);
+.  bfd_byte *  (*_bfd_get_relocated_section_contents) (bfd *,
+.						       struct bfd_link_info *,
+.						       struct bfd_link_order *,
+.						       bfd_byte *, bfd_boolean,
+.						       struct bfd_symbol **);
 .
-.  bfd_boolean (*_bfd_relax_section)
-.    (bfd *, struct bfd_section *, struct bfd_link_info *, bfd_boolean *);
+.  bfd_boolean (*_bfd_relax_section) (bfd *, struct bfd_section *,
+.				      struct bfd_link_info *, bfd_boolean *);
 .
 .  {* Create a hash table for the linker.  Different backends store
 .     different information in this table.  *}
 .  struct bfd_link_hash_table *
-.              (*_bfd_link_hash_table_create) (bfd *);
+.	       (*_bfd_link_hash_table_create) (bfd *);
 .
 .  {* Add symbols from this object file into the hash table.  *}
 .  bfd_boolean (*_bfd_link_add_symbols) (bfd *, struct bfd_link_info *);
 .
 .  {* Indicate that we are only retrieving symbol values from this section.  *}
-.  void        (*_bfd_link_just_syms) (asection *, struct bfd_link_info *);
+.  void	       (*_bfd_link_just_syms) (asection *, struct bfd_link_info *);
 .
 .  {* Copy the symbol type and other attributes for a linker script
 .     assignment of one symbol to another.  *}
 .#define bfd_copy_link_hash_symbol_type(b, t, f) \
-.  BFD_SEND (b, _bfd_copy_link_hash_symbol_type, (b, t, f))
-.  void (*_bfd_copy_link_hash_symbol_type)
-.    (bfd *, struct bfd_link_hash_entry *, struct bfd_link_hash_entry *);
+.	BFD_SEND (b, _bfd_copy_link_hash_symbol_type, (b, t, f))
+.  void	       (*_bfd_copy_link_hash_symbol_type) (bfd *,
+.						   struct bfd_link_hash_entry *,
+.						   struct bfd_link_hash_entry *);
 .
 .  {* Do a link based on the link_order structures attached to each
 .     section of the BFD.  *}
@@ -492,13 +506,15 @@ BFD_JUMP_TABLE macros.
 .  {* Should this section be split up into smaller pieces during linking.  *}
 .  bfd_boolean (*_bfd_link_split_section) (bfd *, struct bfd_section *);
 .
+.  {* Check the relocations in the bfd for validity.  *}
+.  bfd_boolean (* _bfd_link_check_relocs)(bfd *, struct bfd_link_info *);
+.
 .  {* Remove sections that are not referenced from the output.  *}
 .  bfd_boolean (*_bfd_gc_sections) (bfd *, struct bfd_link_info *);
 .
 .  {* Sets the bitmask of allowed and disallowed section flags.  *}
 .  bfd_boolean (*_bfd_lookup_section_flags) (struct bfd_link_info *,
-.					     struct flag_info *,
-.					     asection *);
+.					     struct flag_info *, asection *);
 .
 .  {* Attempt to merge SEC_MERGE sections.  *}
 .  bfd_boolean (*_bfd_merge_sections) (bfd *, struct bfd_link_info *);
@@ -518,6 +534,11 @@ BFD_JUMP_TABLE macros.
 .  bfd_boolean (*_bfd_define_common_symbol) (bfd *, struct bfd_link_info *,
 .					     struct bfd_link_hash_entry *);
 .
+.  {* Define a __start, __stop, .startof. or .sizeof. symbol.  *}
+.  struct bfd_link_hash_entry *
+.	       (*_bfd_define_start_stop) (struct bfd_link_info *, const char *,
+.					  asection *);
+.
 .  {* Routines to handle dynamic symbols and relocs.  *}
 .#define BFD_JUMP_TABLE_DYNAMIC(NAME) \
 .  NAME##_get_dynamic_symtab_upper_bound, \
@@ -527,19 +548,18 @@ BFD_JUMP_TABLE macros.
 .  NAME##_canonicalize_dynamic_reloc
 .
 .  {* Get the amount of memory required to hold the dynamic symbols.  *}
-.  long        (*_bfd_get_dynamic_symtab_upper_bound) (bfd *);
+.  long	       (*_bfd_get_dynamic_symtab_upper_bound) (bfd *);
 .  {* Read in the dynamic symbols.  *}
-.  long        (*_bfd_canonicalize_dynamic_symtab)
-.    (bfd *, struct bfd_symbol **);
+.  long	       (*_bfd_canonicalize_dynamic_symtab) (bfd *, struct bfd_symbol **);
 .  {* Create synthetized symbols.  *}
-.  long        (*_bfd_get_synthetic_symtab)
-.    (bfd *, long, struct bfd_symbol **, long, struct bfd_symbol **,
-.     struct bfd_symbol **);
+.  long	       (*_bfd_get_synthetic_symtab) (bfd *, long, struct bfd_symbol **,
+.					     long, struct bfd_symbol **,
+.					     struct bfd_symbol **);
 .  {* Get the amount of memory required to hold the dynamic relocs.  *}
-.  long        (*_bfd_get_dynamic_reloc_upper_bound) (bfd *);
+.  long	       (*_bfd_get_dynamic_reloc_upper_bound) (bfd *);
 .  {* Read in the dynamic relocs.  *}
-.  long        (*_bfd_canonicalize_dynamic_reloc)
-.    (bfd *, arelent **, struct bfd_symbol **);
+.  long	       (*_bfd_canonicalize_dynamic_reloc) (bfd *, arelent **,
+.						   struct bfd_symbol **);
 .
 
 A pointer to an alternative bfd_target in case the current one is not
@@ -549,7 +569,7 @@ endianness.  The function open_output() in ld/ldlang.c uses this field
 to find an alternative output format that is suitable.
 
 .  {* Opposite endian version of this target.  *}
-.  const struct bfd_target * alternative_target;
+.  const struct bfd_target *alternative_target;
 .
 
 .  {* Data for use by back-end routines, which isn't
@@ -567,7 +587,10 @@ to find an alternative output format that is suitable.
 extern const bfd_target aarch64_elf32_be_vec;
 extern const bfd_target aarch64_elf32_le_vec;
 extern const bfd_target aarch64_elf64_be_vec;
+extern const bfd_target aarch64_elf64_be_cloudabi_vec;
 extern const bfd_target aarch64_elf64_le_vec;
+extern const bfd_target aarch64_elf64_le_cloudabi_vec;
+extern const bfd_target aarch64_mach_o_vec;
 extern const bfd_target alpha_ecoff_le_vec;
 extern const bfd_target alpha_elf64_vec;
 extern const bfd_target alpha_elf64_fbsd_vec;
@@ -595,6 +618,7 @@ extern const bfd_target arm_elf32_symbian_be_vec;
 extern const bfd_target arm_elf32_symbian_le_vec;
 extern const bfd_target arm_elf32_vxworks_be_vec;
 extern const bfd_target arm_elf32_vxworks_le_vec;
+extern const bfd_target arm_mach_o_vec;
 extern const bfd_target arm_pe_be_vec;
 extern const bfd_target arm_pe_le_vec;
 extern const bfd_target arm_pe_epoc_be_vec;
@@ -631,6 +655,7 @@ extern const bfd_target frv_elf32_vec;
 extern const bfd_target frv_elf32_fdpic_vec;
 extern const bfd_target h8300_coff_vec;
 extern const bfd_target h8300_elf32_vec;
+extern const bfd_target h8300_elf32_linux_vec;
 extern const bfd_target h8500_coff_vec;
 extern const bfd_target hppa_elf32_vec;
 extern const bfd_target hppa_elf32_linux_vec;
@@ -662,6 +687,7 @@ extern const bfd_target i386_msdos_vec;
 extern const bfd_target i386_nlm32_vec;
 extern const bfd_target i386_pe_vec;
 extern const bfd_target i386_pei_vec;
+extern const bfd_target iamcu_elf32_vec;
 extern const bfd_target i860_coff_vec;
 extern const bfd_target i860_elf32_vec;
 extern const bfd_target i860_elf32_le_vec;
@@ -788,6 +814,9 @@ extern const bfd_target powerpc_pe_le_vec;
 extern const bfd_target powerpc_pei_vec;
 extern const bfd_target powerpc_pei_le_vec;
 extern const bfd_target powerpc_xcoff_vec;
+extern const bfd_target pru_elf32_vec;
+extern const bfd_target riscv_elf32_vec;
+extern const bfd_target riscv_elf64_vec;
 extern const bfd_target rl78_elf32_vec;
 extern const bfd_target rs6000_xcoff64_vec;
 extern const bfd_target rs6000_xcoff64_aix_vec;
@@ -872,17 +901,21 @@ extern const bfd_target tilegx_elf64_le_vec;
 extern const bfd_target tilepro_elf32_vec;
 extern const bfd_target v800_elf32_vec;
 extern const bfd_target v850_elf32_vec;
+extern const bfd_target ft32_elf32_vec;
 extern const bfd_target vax_aout_1knbsd_vec;
 extern const bfd_target vax_aout_bsd_vec;
 extern const bfd_target vax_aout_nbsd_vec;
 extern const bfd_target vax_elf32_vec;
 extern const bfd_target visium_elf32_vec;
 extern const bfd_target w65_coff_vec;
+extern const bfd_target wasm_vec;
+extern const bfd_target wasm32_elf32_vec;
 extern const bfd_target we32k_coff_vec;
 extern const bfd_target x86_64_coff_vec;
 extern const bfd_target x86_64_elf32_vec;
 extern const bfd_target x86_64_elf32_nacl_vec;
 extern const bfd_target x86_64_elf64_vec;
+extern const bfd_target x86_64_elf64_cloudabi_vec;
 extern const bfd_target x86_64_elf64_fbsd_vec;
 extern const bfd_target x86_64_elf64_nacl_vec;
 extern const bfd_target x86_64_elf64_sol2_vec;
@@ -943,7 +976,10 @@ static const bfd_target * const _bfd_target_vector[] =
 	&aarch64_elf32_be_vec,
 	&aarch64_elf32_le_vec,
 	&aarch64_elf64_be_vec,
+	&aarch64_elf64_be_cloudabi_vec,
 	&aarch64_elf64_le_vec,
+	&aarch64_elf64_le_cloudabi_vec,
+	&aarch64_mach_o_vec,
 #endif
 
 #ifdef BFD64
@@ -989,6 +1025,7 @@ static const bfd_target * const _bfd_target_vector[] =
 	&arm_elf32_symbian_le_vec,
 	&arm_elf32_vxworks_be_vec,
 	&arm_elf32_vxworks_le_vec,
+	&arm_mach_o_vec,
 	&arm_pe_be_vec,
 	&arm_pe_le_vec,
 	&arm_pe_epoc_be_vec,
@@ -1044,6 +1081,7 @@ static const bfd_target * const _bfd_target_vector[] =
 
 	&h8300_coff_vec,
 	&h8300_elf32_vec,
+	&h8300_elf32_linux_vec,
 	&h8500_coff_vec,
 
 	&hppa_elf32_vec,
@@ -1090,6 +1128,8 @@ static const bfd_target * const _bfd_target_vector[] =
 	&i386_pe_vec,
 	&i386_pei_vec,
 
+	&iamcu_elf32_vec,
+
 	&i860_coff_vec,
 	&i860_elf32_vec,
 	&i860_elf32_le_vec,
@@ -1283,6 +1323,12 @@ static const bfd_target * const _bfd_target_vector[] =
 	&powerpc_xcoff_vec,
 #endif
 
+	&pru_elf32_vec,
+
+#ifdef BFD64
+	&riscv_elf32_vec,
+	&riscv_elf64_vec,
+#endif
 	&rl78_elf32_vec,
 
 #ifdef BFD64
@@ -1378,6 +1424,8 @@ static const bfd_target * const _bfd_target_vector[] =
 #endif
 	&tilepro_elf32_vec,
 
+	&ft32_elf32_vec,
+
 	&v800_elf32_vec,
 	&v850_elf32_vec,
 
@@ -1390,6 +1438,9 @@ static const bfd_target * const _bfd_target_vector[] =
 
 	&w65_coff_vec,
 
+	&wasm_vec,
+	&wasm32_elf32_vec,
+
 	&we32k_coff_vec,
 
 #ifdef BFD64
@@ -1397,6 +1448,7 @@ static const bfd_target * const _bfd_target_vector[] =
 	&x86_64_elf32_vec,
 	&x86_64_elf32_nacl_vec,
 	&x86_64_elf64_vec,
+	&x86_64_elf64_cloudabi_vec,
 	&x86_64_elf64_fbsd_vec,
 	&x86_64_elf64_nacl_vec,
 	&x86_64_elf64_sol2_vec,
@@ -1650,11 +1702,11 @@ _bfd_find_arch_match (const char *tname, const char **arch,
       char end_ch = (in_a ? in_a[strlen (tname)] : 0);
 
       if (in_a && (in_a == *arch || in_a[-1] == ':')
-          && end_ch == 0)
-        {
-          *def_target_arch = *arch;
-          return TRUE;
-        }
+	  && end_ch == 0)
+	{
+	  *def_target_arch = *arch;
+	  return TRUE;
+	}
       arch++;
     }
   return FALSE;
@@ -1670,16 +1722,16 @@ SYNOPSIS
 					       int *underscoring,
 					       const char **def_target_arch);
 DESCRIPTION
-        Return a pointer to the transfer vector for the object target
-        named @var{target_name}.  If @var{target_name} is <<NULL>>,
-        choose the one in the environment variable <<GNUTARGET>>; if
-        that is null or not defined, then choose the first entry in the
-        target list.  Passing in the string "default" or setting the
-        environment variable to "default" will cause the first entry in
-        the target list to be returned, and "target_defaulted" will be
-        set in the BFD if @var{abfd} isn't <<NULL>>.  This causes
-        <<bfd_check_format>> to loop over all the targets to find the
-        one that matches the file being read.
+	Return a pointer to the transfer vector for the object target
+	named @var{target_name}.  If @var{target_name} is <<NULL>>,
+	choose the one in the environment variable <<GNUTARGET>>; if
+	that is null or not defined, then choose the first entry in the
+	target list.  Passing in the string "default" or setting the
+	environment variable to "default" will cause the first entry in
+	the target list to be returned, and "target_defaulted" will be
+	set in the BFD if @var{abfd} isn't <<NULL>>.  This causes
+	<<bfd_check_format>> to loop over all the targets to find the
+	one that matches the file being read.
 	If @var{is_bigendian} is not <<NULL>>, then set this value to target's
 	endian mode. True for big-endian, FALSE for little-endian or for
 	invalid target.
@@ -1717,12 +1769,12 @@ bfd_get_target_info (const char *target_name, bfd *abfd,
       const char **arches = bfd_arch_list ();
 
       if (arches && tname)
-        {
-          char *hyp = strchr (tname, '-');
+	{
+	  char *hyp = strchr (tname, '-');
 
-          if (hyp != NULL)
-            {
-              tname = ++hyp;
+	  if (hyp != NULL)
+	    {
+	      tname = ++hyp;
 
 	      /* Make sure we detect architecture names
 		 for triplets like "pe-arm-wince-little".  */
@@ -1745,7 +1797,7 @@ bfd_get_target_info (const char *target_name, bfd *abfd,
 	}
 
       if (arches)
-        free (arches);
+	free (arches);
     }
   return target_vec;
 }
@@ -1792,30 +1844,76 @@ bfd_target_list (void)
 
 /*
 FUNCTION
-	bfd_seach_for_target
+	bfd_iterate_over_targets
 
 SYNOPSIS
-	const bfd_target *bfd_search_for_target
-	  (int (*search_func) (const bfd_target *, void *),
-	   void *);
+	const bfd_target *bfd_iterate_over_targets
+	  (int (*func) (const bfd_target *, void *),
+	   void *data);
 
 DESCRIPTION
-	Return a pointer to the first transfer vector in the list of
-	transfer vectors maintained by BFD that produces a non-zero
-	result when passed to the function @var{search_func}.  The
-	parameter @var{data} is passed, unexamined, to the search
-	function.
+	Call @var{func} for each target in the list of BFD target
+	vectors, passing @var{data} to @var{func}.  Stop iterating if
+	@var{func} returns a non-zero result, and return that target
+	vector.  Return NULL if @var{func} always returns zero.
 */
 
 const bfd_target *
-bfd_search_for_target (int (*search_func) (const bfd_target *, void *),
-		       void *data)
+bfd_iterate_over_targets (int (*func) (const bfd_target *, void *),
+			  void *data)
 {
-  const bfd_target * const *target;
+  const bfd_target *const *target;
 
-  for (target = bfd_target_vector; *target != NULL; target ++)
-    if (search_func (*target, data))
+  for (target = bfd_target_vector; *target != NULL; ++target)
+    if (func (*target, data))
       return *target;
 
   return NULL;
 }
+
+/*
+FUNCTION
+	bfd_flavour_name
+
+SYNOPSIS
+	const char *bfd_flavour_name (enum bfd_flavour flavour);
+
+DESCRIPTION
+	Return the string form of @var{flavour}.
+*/
+
+const char *
+bfd_flavour_name (enum bfd_flavour flavour)
+{
+  switch (flavour)
+    {
+    case bfd_target_unknown_flavour: return "unknown file format";
+    case bfd_target_aout_flavour: return "a.out";
+    case bfd_target_coff_flavour: return "COFF";
+    case bfd_target_ecoff_flavour: return "ECOFF";
+    case bfd_target_xcoff_flavour: return "XCOFF";
+    case bfd_target_elf_flavour: return "ELF";
+    case bfd_target_ieee_flavour: return "IEEE";
+    case bfd_target_nlm_flavour: return "NLM";
+    case bfd_target_oasys_flavour: return "Oasys";
+    case bfd_target_tekhex_flavour: return "Tekhex";
+    case bfd_target_srec_flavour: return "Srec";
+    case bfd_target_verilog_flavour: return "Verilog";
+    case bfd_target_ihex_flavour: return "Ihex";
+    case bfd_target_som_flavour: return "SOM";
+    case bfd_target_os9k_flavour: return "OS9K";
+    case bfd_target_versados_flavour: return "Versados";
+    case bfd_target_msdos_flavour: return "MSDOS";
+    case bfd_target_ovax_flavour: return "Ovax";
+    case bfd_target_evax_flavour: return "Evax";
+    case bfd_target_mmo_flavour: return "mmo";
+    case bfd_target_mach_o_flavour: return "MACH_O";
+    case bfd_target_pef_flavour: return "PEF";
+    case bfd_target_pef_xlib_flavour: return "PEF_XLIB";
+    case bfd_target_sym_flavour: return "SYM";
+    /* There is no "default" case here so that -Wswitch (part of -Wall)
+       catches missing entries.  */
+    }
+
+  abort ();
+}
