@@ -1,5 +1,5 @@
 /* BFD backend for Extended Tektronix Hex Format  objects.
-   Copyright (C) 1992-2015 Free Software Foundation, Inc.
+   Copyright (C) 1992-2017 Free Software Foundation, Inc.
    Written by Steve Chamberlain of Cygnus Support <sac@cygnus.com>.
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -30,7 +30,7 @@
 	devices like PROM programmers and ICE equipment.
 
 	It seems that the sections are described as being really big,
-        the example I have says that the text section is 0..ffffffff.
+	the example I have says that the text section is 0..ffffffff.
 	BFD would barf with this, many apps would try to alloc 4GB to
 	read in the file.
 
@@ -39,7 +39,7 @@
 	one section for each block of data, called "blknnnn" which we
 	stick all the data into.
 
-	TekHex may come out of 	order and there is no header, so an
+	TekHex may come out of order and there is no header, so an
 	initial scan is required  to discover the minimum and maximum
 	addresses used to create the vma and size of the sections we
 	create.
@@ -121,10 +121,10 @@ static char sum_block[256];
    <---------------------- 3a (58 chars) ------------------->
 
    %1B3709T_SEGMENT1108FFFFFFFF
-         ^         ^^ ^- 8 character integer 0xffffffff
-         |         |+-   1 character integer 0
-         |         +--   type 1 symbol (section definition)
-         +------------   9 char symbol T_SEGMENT
+	 ^         ^^ ^- 8 character integer 0xffffffff
+	 |         |+-   1 character integer 0
+	 |         +--   type 1 symbol (section definition)
+	 +------------   9 char symbol T_SEGMENT
 
    %2B3AB9T_SEGMENT7Dgcc_compiled$1087hello$c10
    %373829T_SEGMENT80int$t1$r1$$214741080char$t2$r2$0$12710
@@ -267,19 +267,22 @@ typedef struct tekhex_data_struct
 #define enda(x) (x->vma + x->size)
 
 static bfd_boolean
-getvalue (char **srcp, bfd_vma *valuep)
+getvalue (char **srcp, bfd_vma *valuep, char * endp)
 {
   char *src = *srcp;
   bfd_vma value = 0;
   unsigned int len;
 
+  if (src >= endp)
+    return FALSE;
+
   if (!ISHEX (*src))
     return FALSE;
 
   len = hex_value (*src++);
   if (len == 0)
     len = 16;
-  while (len--)
+  while (len-- && src < endp)
     {
       if (!ISHEX (*src))
 	return FALSE;
@@ -288,11 +291,11 @@ getvalue (char **srcp, bfd_vma *valuep)
 
   *srcp = src;
   *valuep = value;
-  return TRUE;
+  return len == -1U;
 }
 
 static bfd_boolean
-getsym (char *dstp, char **srcp, unsigned int *lenp)
+getsym (char *dstp, char **srcp, unsigned int *lenp, char * endp)
 {
   char *src = *srcp;
   unsigned int i;
@@ -304,12 +307,12 @@ getsym (char *dstp, char **srcp, unsigned int *lenp)
   len = hex_value (*src++);
   if (len == 0)
     len = 16;
-  for (i = 0; i < len; i++)
+  for (i = 0; i < len && (src + i) < endp; i++)
     dstp[i] = src[i];
   dstp[i] = 0;
   *srcp = src + i;
   *lenp = len;
-  return TRUE;
+  return i == len;
 }
 
 static struct data_struct *
@@ -325,7 +328,7 @@ find_chunk (bfd *abfd, bfd_vma vma, bfd_boolean create)
     {
       /* No chunk for this address, so make one up.  */
       d = (struct data_struct *)
-          bfd_zalloc (abfd, (bfd_size_type) sizeof (struct data_struct));
+	  bfd_zalloc (abfd, (bfd_size_type) sizeof (struct data_struct));
 
       if (!d)
 	return NULL;
@@ -354,7 +357,7 @@ insert_byte (bfd *abfd, int value, bfd_vma addr)
   how big the data is.  */
 
 static bfd_boolean
-first_phase (bfd *abfd, int type, char *src)
+first_phase (bfd *abfd, int type, char *src, char * src_end)
 {
   asection *section, *alt_section;
   unsigned int len;
@@ -368,21 +371,21 @@ first_phase (bfd *abfd, int type, char *src)
       {
 	bfd_vma addr;
 
-	if (!getvalue (&src, &addr))
+	if (!getvalue (&src, &addr, src_end))
 	  return FALSE;
 
-	while (*src)
+	while (*src && src < src_end - 1)
 	  {
 	    insert_byte (abfd, HEX (src), addr);
 	    src += 2;
 	    addr++;
 	  }
+	return TRUE;
       }
 
-      return TRUE;
     case '3':
       /* Symbol record, read the segment.  */
-      if (!getsym (sym, &src, &len))
+      if (!getsym (sym, &src, &len, src_end))
 	return FALSE;
       section = bfd_get_section_by_name (abfd, sym);
       if (section == NULL)
@@ -397,17 +400,23 @@ first_phase (bfd *abfd, int type, char *src)
 	    return FALSE;
 	}
       alt_section = NULL;
-      while (*src)
+      while (src < src_end && *src)
 	{
 	  switch (*src)
 	    {
 	    case '1':		/* Section range.  */
 	      src++;
-	      if (!getvalue (&src, &section->vma))
+	      if (!getvalue (&src, &section->vma, src_end))
 		return FALSE;
-	      if (!getvalue (&src, &val))
+	      if (!getvalue (&src, &val, src_end))
 		return FALSE;
+	      if (val < section->vma)
+		val = section->vma;
 	      section->size = val - section->vma;
+	      /* PR 17512: file: objdump-s-endless-loop.tekhex.
+		 Check for overlarge section sizes.  */
+	      if (section->size & 0x80000000)
+		return FALSE;
 	      section->flags = SEC_HAS_CONTENTS | SEC_LOAD | SEC_ALLOC;
 	      break;
 	    case '0':
@@ -421,7 +430,7 @@ first_phase (bfd *abfd, int type, char *src)
 	      {
 		bfd_size_type amt = sizeof (tekhex_symbol_type);
 		tekhex_symbol_type *new_symbol = (tekhex_symbol_type *)
-                    bfd_alloc (abfd, amt);
+		    bfd_alloc (abfd, amt);
 		char stype = (*src);
 
 		if (!new_symbol)
@@ -432,10 +441,10 @@ first_phase (bfd *abfd, int type, char *src)
 		abfd->flags |= HAS_SYMS;
 		new_symbol->prev = abfd->tdata.tekhex_data->symbols;
 		abfd->tdata.tekhex_data->symbols = new_symbol;
-		if (!getsym (sym, &src, &len))
+		if (!getsym (sym, &src, &len, src_end))
 		  return FALSE;
 		new_symbol->symbol.name = (const char *)
-                    bfd_alloc (abfd, (bfd_size_type) len + 1);
+		    bfd_alloc (abfd, (bfd_size_type) len + 1);
 		if (!new_symbol->symbol.name)
 		  return FALSE;
 		memcpy ((char *) (new_symbol->symbol.name), sym, len + 1);
@@ -453,7 +462,8 @@ first_phase (bfd *abfd, int type, char *src)
 		    else
 		      {
 			if (alt_section == NULL)
-			  alt_section = bfd_get_next_section_by_name (section);
+			  alt_section
+			    = bfd_get_next_section_by_name (NULL, section);
 			if (alt_section == NULL)
 			  alt_section = bfd_make_section_anyway_with_flags
 			    (abfd, section->name,
@@ -470,7 +480,8 @@ first_phase (bfd *abfd, int type, char *src)
 		    else
 		      {
 			if (alt_section == NULL)
-			  alt_section = bfd_get_next_section_by_name (section);
+			  alt_section
+			    = bfd_get_next_section_by_name (NULL, section);
 			if (alt_section == NULL)
 			  alt_section = bfd_make_section_anyway_with_flags
 			    (abfd, section->name,
@@ -480,7 +491,7 @@ first_phase (bfd *abfd, int type, char *src)
 			new_symbol->symbol.section = alt_section;
 		      }
 		  }
-		if (!getvalue (&src, &val))
+		if (!getvalue (&src, &val, src_end))
 		  return FALSE;
 		new_symbol->symbol.value = val - section->vma;
 		break;
@@ -498,7 +509,7 @@ first_phase (bfd *abfd, int type, char *src)
    record.  */
 
 static bfd_boolean
-pass_over (bfd *abfd, bfd_boolean (*func) (bfd *, int, char *))
+pass_over (bfd *abfd, bfd_boolean (*func) (bfd *, int, char *, char *))
 {
   unsigned int chars_on_line;
   bfd_boolean is_eof = FALSE;
@@ -506,6 +517,7 @@ pass_over (bfd *abfd, bfd_boolean (*func) (bfd *, int, char *))
   /* To the front of the file.  */
   if (bfd_seek (abfd, (file_ptr) 0, SEEK_SET) != 0)
     return FALSE;
+
   while (! is_eof)
     {
       char src[MAXCHUNK];
@@ -539,8 +551,7 @@ pass_over (bfd *abfd, bfd_boolean (*func) (bfd *, int, char *))
 
       /* Put a null at the end.  */
       src[chars_on_line] = 0;
-
-      if (!func (abfd, type, src))
+      if (!func (abfd, type, src, src + chars_on_line))
 	return FALSE;
     }
 
@@ -903,7 +914,7 @@ tekhex_make_empty_symbol (bfd *abfd)
 {
   bfd_size_type amt = sizeof (struct tekhex_symbol_struct);
   tekhex_symbol_type *new_symbol = (tekhex_symbol_type *) bfd_zalloc (abfd,
-                                                                      amt);
+								      amt);
 
   if (!new_symbol)
     return NULL;
@@ -948,36 +959,37 @@ tekhex_print_symbol (bfd *abfd,
     }
 }
 
-#define	tekhex_close_and_cleanup                    _bfd_generic_close_and_cleanup
-#define tekhex_bfd_free_cached_info                 _bfd_generic_bfd_free_cached_info
-#define tekhex_new_section_hook                     _bfd_generic_new_section_hook
+#define	tekhex_close_and_cleanup		    _bfd_generic_close_and_cleanup
+#define tekhex_bfd_free_cached_info		    _bfd_generic_bfd_free_cached_info
+#define tekhex_new_section_hook			    _bfd_generic_new_section_hook
 #define tekhex_bfd_is_target_special_symbol ((bfd_boolean (*) (bfd *, asymbol *)) bfd_false)
-#define tekhex_bfd_is_local_label_name               bfd_generic_is_local_label_name
-#define tekhex_get_lineno                           _bfd_nosymbols_get_lineno
-#define tekhex_find_nearest_line                    _bfd_nosymbols_find_nearest_line
-#define tekhex_find_line                            _bfd_nosymbols_find_line
-#define tekhex_find_inliner_info                    _bfd_nosymbols_find_inliner_info
+#define tekhex_bfd_is_local_label_name		     bfd_generic_is_local_label_name
+#define tekhex_get_lineno			    _bfd_nosymbols_get_lineno
+#define tekhex_find_nearest_line		    _bfd_nosymbols_find_nearest_line
+#define tekhex_find_line			    _bfd_nosymbols_find_line
+#define tekhex_find_inliner_info		    _bfd_nosymbols_find_inliner_info
 #define tekhex_get_symbol_version_string	    _bfd_nosymbols_get_symbol_version_string
-#define tekhex_bfd_make_debug_symbol                _bfd_nosymbols_bfd_make_debug_symbol
-#define tekhex_read_minisymbols                     _bfd_generic_read_minisymbols
-#define tekhex_minisymbol_to_symbol                 _bfd_generic_minisymbol_to_symbol
+#define tekhex_bfd_make_debug_symbol		    _bfd_nosymbols_bfd_make_debug_symbol
+#define tekhex_read_minisymbols			    _bfd_generic_read_minisymbols
+#define tekhex_minisymbol_to_symbol		    _bfd_generic_minisymbol_to_symbol
 #define tekhex_bfd_get_relocated_section_contents   bfd_generic_get_relocated_section_contents
-#define tekhex_bfd_relax_section                    bfd_generic_relax_section
-#define tekhex_bfd_gc_sections                      bfd_generic_gc_sections
+#define tekhex_bfd_relax_section		    bfd_generic_relax_section
+#define tekhex_bfd_gc_sections			    bfd_generic_gc_sections
 #define tekhex_bfd_lookup_section_flags		    bfd_generic_lookup_section_flags
-#define tekhex_bfd_merge_sections                   bfd_generic_merge_sections
-#define tekhex_bfd_is_group_section                 bfd_generic_is_group_section
-#define tekhex_bfd_discard_group                    bfd_generic_discard_group
-#define tekhex_section_already_linked               _bfd_generic_section_already_linked
-#define tekhex_bfd_define_common_symbol             bfd_generic_define_common_symbol
-#define tekhex_bfd_link_hash_table_create           _bfd_generic_link_hash_table_create
-#define tekhex_bfd_link_add_symbols                 _bfd_generic_link_add_symbols
-#define tekhex_bfd_link_just_syms                   _bfd_generic_link_just_syms
-#define tekhex_bfd_copy_link_hash_symbol_type \
-  _bfd_generic_copy_link_hash_symbol_type
-#define tekhex_bfd_final_link                       _bfd_generic_final_link
-#define tekhex_bfd_link_split_section               _bfd_generic_link_split_section
-#define tekhex_get_section_contents_in_window       _bfd_generic_get_section_contents_in_window
+#define tekhex_bfd_merge_sections		    bfd_generic_merge_sections
+#define tekhex_bfd_is_group_section		    bfd_generic_is_group_section
+#define tekhex_bfd_discard_group		    bfd_generic_discard_group
+#define tekhex_section_already_linked		    _bfd_generic_section_already_linked
+#define tekhex_bfd_define_common_symbol		    bfd_generic_define_common_symbol
+#define tekhex_bfd_define_start_stop		    bfd_generic_define_start_stop
+#define tekhex_bfd_link_hash_table_create	    _bfd_generic_link_hash_table_create
+#define tekhex_bfd_link_add_symbols		    _bfd_generic_link_add_symbols
+#define tekhex_bfd_link_just_syms		    _bfd_generic_link_just_syms
+#define tekhex_bfd_copy_link_hash_symbol_type	    _bfd_generic_copy_link_hash_symbol_type
+#define tekhex_bfd_final_link			    _bfd_generic_final_link
+#define tekhex_bfd_link_split_section		    _bfd_generic_link_split_section
+#define tekhex_get_section_contents_in_window	    _bfd_generic_get_section_contents_in_window
+#define tekhex_bfd_link_check_relocs		    _bfd_generic_link_check_relocs
 
 const bfd_target tekhex_vec =
 {
