@@ -19,6 +19,9 @@
 #define COMMAND_H 1
 
 #include "gdb_vecs.h"
+#include "common/scoped_restore.h"
+
+struct completion_tracker;
 
 /* This file defines the public interface for any code wanting to
    create commands.  */
@@ -113,7 +116,7 @@ var_types;
 /* This structure records one command'd definition.  */
 struct cmd_list_element;
 
-typedef void cmd_cfunc_ftype (char *args, int from_tty);
+typedef void cmd_const_cfunc_ftype (const char *args, int from_tty);
 
 /* This structure specifies notifications to be suppressed by a cli
    command interpreter.  */
@@ -132,8 +135,16 @@ extern struct cli_suppress_notification cli_suppress_notification;
 
 extern int valid_user_defined_cmd_name_p (const char *name);
 
+/* Const-correct variant of the above.  */
+
+extern struct cmd_list_element *add_cmd (const char *, enum command_class,
+					 cmd_const_cfunc_ftype *fun,
+					 const char *,
+					 struct cmd_list_element **);
+
+/* Like add_cmd, but no command function is specified.  */
+
 extern struct cmd_list_element *add_cmd (const char *, enum command_class,
-					 cmd_cfunc_ftype *fun,
 					 const char *,
 					 struct cmd_list_element **);
 
@@ -141,8 +152,14 @@ extern struct cmd_list_element *add_alias_cmd (const char *, const char *,
 					       enum command_class, int,
 					       struct cmd_list_element **);
 
+extern struct cmd_list_element *add_alias_cmd (const char *,
+					       cmd_list_element *,
+					       enum command_class, int,
+					       struct cmd_list_element **);
+
+
 extern struct cmd_list_element *add_prefix_cmd (const char *, enum command_class,
-						cmd_cfunc_ftype *fun,
+						cmd_const_cfunc_ftype *fun,
 						const char *,
 						struct cmd_list_element **,
 						const char *, int,
@@ -150,40 +167,47 @@ extern struct cmd_list_element *add_prefix_cmd (const char *, enum command_class
 
 extern struct cmd_list_element *add_abbrev_prefix_cmd (const char *,
 						       enum command_class,
-						       cmd_cfunc_ftype *fun,
+						       cmd_const_cfunc_ftype *fun,
 						       const char *,
 						       struct cmd_list_element
 						       **, const char *, int,
 						       struct cmd_list_element
 						       **);
 
-/* Set the commands corresponding callback.  */
-
-extern void set_cmd_cfunc (struct cmd_list_element *cmd,
-			   cmd_cfunc_ftype *cfunc);
-
-typedef void cmd_sfunc_ftype (char *args, int from_tty,
-			      struct cmd_list_element *c);
+typedef void cmd_const_sfunc_ftype (const char *args, int from_tty,
+				    struct cmd_list_element *c);
 extern void set_cmd_sfunc (struct cmd_list_element *cmd,
-			   cmd_sfunc_ftype *sfunc);
+			   cmd_const_sfunc_ftype *sfunc);
 
-typedef VEC (char_ptr) *completer_ftype (struct cmd_list_element *,
-					 const char *, const char *);
+/* A completion routine.  Add possible completions to tracker.
 
-typedef void completer_ftype_void (struct cmd_list_element *,
-				   const char *, const char *);
+   TEXT is the text beyond what was matched for the command itself
+   (leading whitespace is skipped).  It stops where we are supposed to
+   stop completing (rl_point) and is '\0' terminated.  WORD points in
+   the same buffer as TEXT, and completions should be returned
+   relative to this position.  For example, suppose TEXT is "foo" and
+   we want to complete to "foobar".  If WORD is "oo", return "oobar";
+   if WORD is "baz/foo", return "baz/foobar".  */
+typedef void completer_ftype (struct cmd_list_element *,
+			      completion_tracker &tracker,
+			      const char *text, const char *word);
+
+/* Same, but for set_cmd_completer_handle_brkchars.  */
+typedef void completer_handle_brkchars_ftype (struct cmd_list_element *,
+					      completion_tracker &tracker,
+					      const char *text, const char *word);
 
 extern void set_cmd_completer (struct cmd_list_element *, completer_ftype *);
 
 /* Set the completer_handle_brkchars callback.  */
 
 extern void set_cmd_completer_handle_brkchars (struct cmd_list_element *,
-					       completer_ftype_void *);
+					       completer_handle_brkchars_ftype *);
 
 /* HACK: cagney/2002-02-23: Code, mostly in tracepoints.c, grubs
    around in cmd objects to test the value of the commands sfunc().  */
 extern int cmd_cfunc_eq (struct cmd_list_element *cmd,
-			 cmd_cfunc_ftype *cfun);
+			 cmd_const_cfunc_ftype *cfun);
 
 /* Each command object has a local context attached to it.  */
 extern void set_cmd_context (struct cmd_list_element *cmd,
@@ -224,7 +248,7 @@ extern int lookup_cmd_composition (const char *text,
 				   struct cmd_list_element **cmd);
 
 extern struct cmd_list_element *add_com (const char *, enum command_class,
-					 cmd_cfunc_ftype *fun,
+					 cmd_const_cfunc_ftype *fun,
 					 const char *);
 
 extern struct cmd_list_element *add_com_alias (const char *, const char *,
@@ -232,21 +256,23 @@ extern struct cmd_list_element *add_com_alias (const char *, const char *,
 
 extern struct cmd_list_element *add_com_suppress_notification
 		       (const char *name, enum command_class theclass,
-			cmd_cfunc_ftype *fun, const char *doc,
+			cmd_const_cfunc_ftype *fun, const char *doc,
 			int *supress_notification);
 
 extern struct cmd_list_element *add_info (const char *,
-					  cmd_cfunc_ftype *fun,
+					  cmd_const_cfunc_ftype *fun,
 					  const char *);
 
 extern struct cmd_list_element *add_info_alias (const char *, const char *,
 						int);
 
-extern VEC (char_ptr) *complete_on_cmdlist (struct cmd_list_element *,
-					    const char *, const char *, int);
+extern void complete_on_cmdlist (struct cmd_list_element *,
+				 completion_tracker &tracker,
+				 const char *, const char *, int);
 
-extern VEC (char_ptr) *complete_on_enum (const char *const *enumlist,
-					 const char *, const char *);
+extern void complete_on_enum (completion_tracker &tracker,
+			      const char *const *enumlist,
+			      const char *, const char *);
 
 /* Functions that implement commands about CLI commands.  */
 
@@ -271,7 +297,7 @@ extern void add_setshow_enum_cmd (const char *name,
 				  const char *set_doc,
 				  const char *show_doc,
 				  const char *help_doc,
-				  cmd_sfunc_ftype *set_func,
+				  cmd_const_sfunc_ftype *set_func,
 				  show_value_ftype *show_func,
 				  struct cmd_list_element **set_list,
 				  struct cmd_list_element **show_list);
@@ -282,7 +308,7 @@ extern void add_setshow_auto_boolean_cmd (const char *name,
 					  const char *set_doc,
 					  const char *show_doc,
 					  const char *help_doc,
-					  cmd_sfunc_ftype *set_func,
+					  cmd_const_sfunc_ftype *set_func,
 					  show_value_ftype *show_func,
 					  struct cmd_list_element **set_list,
 					  struct cmd_list_element **show_list);
@@ -292,7 +318,7 @@ extern void add_setshow_boolean_cmd (const char *name,
 				     int *var,
 				     const char *set_doc, const char *show_doc,
 				     const char *help_doc,
-				     cmd_sfunc_ftype *set_func,
+				     cmd_const_sfunc_ftype *set_func,
 				     show_value_ftype *show_func,
 				     struct cmd_list_element **set_list,
 				     struct cmd_list_element **show_list);
@@ -303,7 +329,7 @@ extern void add_setshow_filename_cmd (const char *name,
 				      const char *set_doc,
 				      const char *show_doc,
 				      const char *help_doc,
-				      cmd_sfunc_ftype *set_func,
+				      cmd_const_sfunc_ftype *set_func,
 				      show_value_ftype *show_func,
 				      struct cmd_list_element **set_list,
 				      struct cmd_list_element **show_list);
@@ -314,7 +340,7 @@ extern void add_setshow_string_cmd (const char *name,
 				    const char *set_doc,
 				    const char *show_doc,
 				    const char *help_doc,
-				    cmd_sfunc_ftype *set_func,
+				    cmd_const_sfunc_ftype *set_func,
 				    show_value_ftype *show_func,
 				    struct cmd_list_element **set_list,
 				    struct cmd_list_element **show_list);
@@ -326,7 +352,7 @@ extern struct cmd_list_element *add_setshow_string_noescape_cmd
 		       const char *set_doc,
 		       const char *show_doc,
 		       const char *help_doc,
-		       cmd_sfunc_ftype *set_func,
+		       cmd_const_sfunc_ftype *set_func,
 		       show_value_ftype *show_func,
 		       struct cmd_list_element **set_list,
 		       struct cmd_list_element **show_list);
@@ -337,7 +363,7 @@ extern void add_setshow_optional_filename_cmd (const char *name,
 					       const char *set_doc,
 					       const char *show_doc,
 					       const char *help_doc,
-					       cmd_sfunc_ftype *set_func,
+					       cmd_const_sfunc_ftype *set_func,
 					       show_value_ftype *show_func,
 					       struct cmd_list_element **set_list,
 					       struct cmd_list_element **show_list);
@@ -348,7 +374,7 @@ extern void add_setshow_integer_cmd (const char *name,
 				     const char *set_doc,
 				     const char *show_doc,
 				     const char *help_doc,
-				     cmd_sfunc_ftype *set_func,
+				     cmd_const_sfunc_ftype *set_func,
 				     show_value_ftype *show_func,
 				     struct cmd_list_element **set_list,
 				     struct cmd_list_element **show_list);
@@ -359,7 +385,7 @@ extern void add_setshow_uinteger_cmd (const char *name,
 				      const char *set_doc,
 				      const char *show_doc,
 				      const char *help_doc,
-				      cmd_sfunc_ftype *set_func,
+				      cmd_const_sfunc_ftype *set_func,
 				      show_value_ftype *show_func,
 				      struct cmd_list_element **set_list,
 				      struct cmd_list_element **show_list);
@@ -370,7 +396,7 @@ extern void add_setshow_zinteger_cmd (const char *name,
 				      const char *set_doc,
 				      const char *show_doc,
 				      const char *help_doc,
-				      cmd_sfunc_ftype *set_func,
+				      cmd_const_sfunc_ftype *set_func,
 				      show_value_ftype *show_func,
 				      struct cmd_list_element **set_list,
 				      struct cmd_list_element **show_list);
@@ -381,7 +407,7 @@ extern void add_setshow_zuinteger_cmd (const char *name,
 				       const char *set_doc,
 				       const char *show_doc,
 				       const char *help_doc,
-				       cmd_sfunc_ftype *set_func,
+				       cmd_const_sfunc_ftype *set_func,
 				       show_value_ftype *show_func,
 				       struct cmd_list_element **set_list,
 				       struct cmd_list_element **show_list);
@@ -393,7 +419,7 @@ extern void
 				       const char *set_doc,
 				       const char *show_doc,
 				       const char *help_doc,
-				       cmd_sfunc_ftype *set_func,
+				       cmd_const_sfunc_ftype *set_func,
 				       show_value_ftype *show_func,
 				       struct cmd_list_element **set_list,
 				       struct cmd_list_element **show_list);
@@ -411,17 +437,22 @@ extern void dont_repeat (void);
 
 extern scoped_restore_tmpl<int> prevent_dont_repeat (void);
 
+/* Set the arguments that will be passed if the current command is
+   repeated.  Note that the passed-in string must be a constant.  */
+
+extern void set_repeat_arguments (const char *args);
+
 /* Used to mark commands that don't do anything.  If we just leave the
    function field NULL, the command is interpreted as a help topic, or
    as a class of commands.  */
 
-extern void not_just_help_class_command (char *, int);
+extern void not_just_help_class_command (const char *, int);
 
 /* Check function pointer.  */
 extern int cmd_func_p (struct cmd_list_element *cmd);
 
 /* Call the command function.  */
 extern void cmd_func (struct cmd_list_element *cmd,
-		      char *args, int from_tty);
+		      const char *args, int from_tty);
 
 #endif /* !defined (COMMAND_H) */
