@@ -1,6 +1,6 @@
 /* TUI Interpreter definitions for GDB, the GNU debugger.
 
-   Copyright (C) 2003-2016 Free Software Foundation, Inc.
+   Copyright (C) 2003-2017 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -34,20 +34,32 @@
 #include "observer.h"
 #include "gdbthread.h"
 
-static struct ui_out *tui_ui_out (struct interp *self);
-
 /* Set to 1 when the TUI mode must be activated when we first start
    gdb.  */
 static int tui_start_enabled = 0;
 
+class tui_interp final : public cli_interp_base
+{
+public:
+  explicit tui_interp (const char *name)
+    : cli_interp_base (name)
+  {}
+
+  void init (bool top_level) override;
+  void resume () override;
+  void suspend () override;
+  gdb_exception exec (const char *command_str) override;
+  ui_out *interp_ui_out () override;
+};
+
 /* Returns the INTERP if the INTERP is a TUI, and returns NULL
    otherwise.  */
 
-static struct interp *
+static tui_interp *
 as_tui_interp (struct interp *interp)
 {
   if (strcmp (interp_name (interp), INTERP_TUI) == 0)
-    return interp;
+    return (tui_interp *) interp;
   return NULL;
 }
 
@@ -70,12 +82,10 @@ tui_exit (void)
 static void
 tui_on_normal_stop (struct bpstats *bs, int print_frame)
 {
-  struct switch_thru_all_uis state;
-
   if (!print_frame)
     return;
 
-  SWITCH_THRU_ALL_UIS (state)
+  SWITCH_THRU_ALL_UIS ()
     {
       struct interp *interp = top_level_interpreter ();
       struct interp *tui = as_tui_interp (interp);
@@ -86,7 +96,7 @@ tui_on_normal_stop (struct bpstats *bs, int print_frame)
 
       thread = inferior_thread ();
       if (should_print_stop_to_console (interp, thread))
-	print_stop_event (tui_ui_out (tui));
+	print_stop_event (tui->interp_ui_out ());
     }
 }
 
@@ -95,16 +105,14 @@ tui_on_normal_stop (struct bpstats *bs, int print_frame)
 static void
 tui_on_signal_received (enum gdb_signal siggnal)
 {
-  struct switch_thru_all_uis state;
-
-  SWITCH_THRU_ALL_UIS (state)
+  SWITCH_THRU_ALL_UIS ()
     {
       struct interp *tui = as_tui_interp (top_level_interpreter ());
 
       if (tui == NULL)
 	continue;
 
-      print_signal_received_reason (tui_ui_out (tui), siggnal);
+      print_signal_received_reason (tui->interp_ui_out (), siggnal);
     }
 }
 
@@ -113,16 +121,14 @@ tui_on_signal_received (enum gdb_signal siggnal)
 static void
 tui_on_end_stepping_range (void)
 {
-  struct switch_thru_all_uis state;
-
-  SWITCH_THRU_ALL_UIS (state)
+  SWITCH_THRU_ALL_UIS ()
     {
       struct interp *tui = as_tui_interp (top_level_interpreter ());
 
       if (tui == NULL)
 	continue;
 
-      print_end_stepping_range_reason (tui_ui_out (tui));
+      print_end_stepping_range_reason (tui->interp_ui_out ());
     }
 }
 
@@ -131,16 +137,14 @@ tui_on_end_stepping_range (void)
 static void
 tui_on_signal_exited (enum gdb_signal siggnal)
 {
-  struct switch_thru_all_uis state;
-
-  SWITCH_THRU_ALL_UIS (state)
+  SWITCH_THRU_ALL_UIS ()
     {
       struct interp *tui = as_tui_interp (top_level_interpreter ());
 
       if (tui == NULL)
 	continue;
 
-      print_signal_exited_reason (tui_ui_out (tui), siggnal);
+      print_signal_exited_reason (tui->interp_ui_out (), siggnal);
     }
 }
 
@@ -149,16 +153,14 @@ tui_on_signal_exited (enum gdb_signal siggnal)
 static void
 tui_on_exited (int exitstatus)
 {
-  struct switch_thru_all_uis state;
-
-  SWITCH_THRU_ALL_UIS (state)
+  SWITCH_THRU_ALL_UIS ()
     {
       struct interp *tui = as_tui_interp (top_level_interpreter ());
 
       if (tui == NULL)
 	continue;
 
-      print_exited_reason (tui_ui_out (tui), exitstatus);
+      print_exited_reason (tui->interp_ui_out (), exitstatus);
     }
 }
 
@@ -167,16 +169,14 @@ tui_on_exited (int exitstatus)
 static void
 tui_on_no_history (void)
 {
-  struct switch_thru_all_uis state;
-
-  SWITCH_THRU_ALL_UIS (state)
+  SWITCH_THRU_ALL_UIS ()
     {
       struct interp *tui = as_tui_interp (top_level_interpreter ());
 
       if (tui == NULL)
 	continue;
 
-      print_no_history_reason (tui_ui_out (tui));
+      print_no_history_reason (tui->interp_ui_out ());
     }
 }
 
@@ -206,10 +206,40 @@ tui_on_command_error (void)
   display_gdb_prompt (NULL);
 }
 
+/* Observer for the user_selected_context_changed notification.  */
+
+static void
+tui_on_user_selected_context_changed (user_selected_what selection)
+{
+  struct thread_info *tp;
+
+  /* This event is suppressed.  */
+  if (cli_suppress_notification.user_selected_context)
+    return;
+
+  tp = find_thread_ptid (inferior_ptid);
+
+  SWITCH_THRU_ALL_UIS ()
+    {
+      struct interp *tui = as_tui_interp (top_level_interpreter ());
+
+      if (tui == NULL)
+	continue;
+
+      if (selection & USER_SELECTED_INFERIOR)
+	print_selected_inferior (tui->interp_ui_out ());
+
+      if (tp != NULL
+	  && ((selection & (USER_SELECTED_THREAD | USER_SELECTED_FRAME))))
+	print_selected_thread_frame (tui->interp_ui_out (), selection);
+
+    }
+}
+
 /* These implement the TUI interpreter.  */
 
-static void *
-tui_init (struct interp *self, int top_level)
+void
+tui_interp::init (bool top_level)
 {
   /* Install exit handler to leave the screen in a good shape.  */
   atexit (tui_exit);
@@ -220,12 +250,10 @@ tui_init (struct interp *self, int top_level)
   tui_initialize_win ();
   if (ui_file_isatty (gdb_stdout))
     tui_initialize_readline ();
-
-  return NULL;
 }
 
-static int
-tui_resume (void *data)
+void
+tui_interp::resume ()
 {
   struct ui *ui = current_ui;
   struct ui_file *stream;
@@ -234,10 +262,10 @@ tui_resume (void *data)
      previously writing to gdb_stdout, then set it to the new
      gdb_stdout afterwards.  */
 
-  stream = cli_out_set_stream (tui_old_uiout, gdb_stdout);
+  stream = tui_old_uiout->set_stream (gdb_stdout);
   if (stream != gdb_stdout)
     {
-      cli_out_set_stream (tui_old_uiout, stream);
+      tui_old_uiout->set_stream (stream);
       stream = NULL;
     }
 
@@ -246,23 +274,21 @@ tui_resume (void *data)
   ui->input_handler = command_line_handler;
 
   if (stream != NULL)
-    cli_out_set_stream (tui_old_uiout, gdb_stdout);
+    tui_old_uiout->set_stream (gdb_stdout);
 
   if (tui_start_enabled)
     tui_enable ();
-  return 1;
 }
 
-static int
-tui_suspend (void *data)
+void
+tui_interp::suspend ()
 {
   tui_start_enabled = tui_active;
   tui_disable ();
-  return 1;
 }
 
-static struct ui_out *
-tui_ui_out (struct interp *self)
+ui_out *
+tui_interp::interp_ui_out ()
 {
   if (tui_active)
     return tui_out;
@@ -270,36 +296,21 @@ tui_ui_out (struct interp *self)
     return tui_old_uiout;
 }
 
-static struct gdb_exception
-tui_exec (void *data, const char *command_str)
+gdb_exception
+tui_interp::exec (const char *command_str)
 {
   internal_error (__FILE__, __LINE__, _("tui_exec called"));
 }
 
-/* The TUI interpreter's vtable.  */
-
-static const struct interp_procs tui_interp_procs = {
-  tui_init,
-  tui_resume,
-  tui_suspend,
-  tui_exec,
-  tui_ui_out,
-  NULL,
-  cli_interpreter_pre_command_loop,
-  cli_interpreter_supports_command_editing,
-};
 
 /* Factory for TUI interpreters.  */
 
 static struct interp *
 tui_interp_factory (const char *name)
 {
-  return interp_new (name, &tui_interp_procs, NULL);
+  return new tui_interp (name);
 }
 
-/* Provide a prototype to silence -Wmissing-prototypes.  */
-extern initialize_file_ftype _initialize_tui_interp;
-
 void
 _initialize_tui_interp (void)
 {
@@ -323,4 +334,6 @@ _initialize_tui_interp (void)
   observer_attach_no_history (tui_on_no_history);
   observer_attach_sync_execution_done (tui_on_sync_execution_done);
   observer_attach_command_error (tui_on_command_error);
+  observer_attach_user_selected_context_changed
+    (tui_on_user_selected_context_changed);
 }
